//-------+---------+---------+---------+---------+---------+---------+--------=
//
// File:	ranges.h								Author:		Bob Harris
//
//----------
//
// Interface definitions for C support module
//
//----------

#ifndef ranges_H
#define ranges_H

#include <stdlib.h>

//----------
//
// Range vectors
//
// (for details, see the description at the top of ranges.c)
//
//----------

typedef struct
	{
	unsigned long	start;	// index of first element in range (origin 1)
	unsigned long	end;	// index of final element in range (inclusive)
							// .. invariant: end >= start
	char			chr;	// chromosome, encoded as per str_to_chromosome();
							// .. if the chromosome is not applicable, zero is
							// .. used here;  this is guaranteed to be in the
							// .. range 0..127
	char			strand;	// direction of range
							// .. '+' => forward strand
							// .. '-' => reverse strand (from end thru start)
							// .. 0   => both strands (or no direction)
	char*			extra;  // any extra info;  for example, when ranges are
							// .. read from a file, any extra stuff on the
							// .. end of the line gets copied here
	long			info;	// any info the user programmer wants
	void*			infoPtr;// any more info the user programmer wants
	} range_t;

typedef struct
	{
	char*		 header;	// the header (or name) of this list;  NULL if it
							// .. has no header
    range_t*	 vec;		// dynamically allocated list of ranges
    unsigned int len;		// the number of entries in v that are in use
    unsigned int max;		// the number of entries allocated for v
	void*		 ((*alloc)(void*, size_t));
    void		 (*free)(void*);
	} ranges_t;

#define ranges_INIT(a,f)  { NULL, NULL, 0, 0, a, f }

//----------
//
// prototypes for routines in ranges.c
//
//----------

void  ranges_fini         (ranges_t* r);
void  range_fini          (range_t* ent);
int   ranges_need         (ranges_t* r, unsigned int howMany);
int   ranges_need_exact   (ranges_t* r, unsigned int howMany);
int   ranges_more         (ranges_t* r, unsigned int howMany);
int   ranges_append       (ranges_t* r, range_t* newRange);
int   ranges_fit          (ranges_t* r);
void  ranges_readfile     (const char* fname, ranges_t* r);
void  ranges_read         (FILE* f, ranges_t* r);
int   parse_range         (char* s, range_t* ent, int lineNumber);
void  ranges_fprintf      (FILE* f, ranges_t* r, char* format);
void  ranges_fprintf_info (FILE* f, ranges_t* r, char* format);
void  range_fprintf       (FILE* f, range_t* ent, char* format);
void  range_fprintf_info  (FILE* f, range_t* ent, char* format);
void  ranges_sort         (ranges_t* r, int (*compar)(const void*, const void*));
int   ranges_ascending    (const void* n1, const void* n2);
void  ranges_exclude      (ranges_t* r, ranges_t* exclude);
int   range_in_ranges     (range_t* rng, ranges_t* r);
int   ranges_in_interval  (ranges_t* r, char chr,
                           unsigned long firstIndex, unsigned long lastIndex,
                           char strand);
int   first_range_in_interval
                          (ranges_t* r, int rIx,
                           char chr,
                           unsigned long firstIndex, unsigned long lastIndex,
                           char strand);
int   ranges_overlap_range(range_t* rng, ranges_t* r);
int   ranges_overlap_interval
                          (ranges_t* r, char chr,
                           unsigned long firstIndex, unsigned long lastIndex,
                           char strand);

#endif // ranges_H
